<?php

namespace App\Service;

use App\Exception;

class ProductService {
    
    private $util;
    private $factory;
    private $combinedConfTela = [];
    private $combinedProducts = [];
    private $combinedChilds = [];
    private $hasCombinedProd = false;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $factory) {

        $this->util    = $util;
        $this->factory = $factory;
    }
    
    public function setHasCombinedProduct($flag) {
        $this->hasCombinedProduct = $flag;
    }
    
    public function getHasCombinedProduct() {
        return $this->hasCombinedProduct;
    }
    
    private function isCombinedProduct($subItems) {
        foreach($subItems as $item) {
            if(strtoupper(substr($item['name'], 0, 5)) === 'MASSA') {
                return true;
            }
        }
        return false;
    }
    
    private function getCombinedProducts($CDFILIAL, $CDCAIXA, $CDLOJA) {
        if (empty($this->combinedConfTela)) {
            $this->combinedConfTela = $this->factory->getCombinedConfTela($CDFILIAL, $CDCAIXA);
        }
        
        if (empty($this->combinedProducts)) {
            $this->combinedProducts = $this->factory->getCombinedProducts($CDFILIAL, $CDLOJA, $this->combinedConfTela);
        }
        
        if (empty($this->combinedChilds)) {
            $this->combinedChilds = $this->factory->getCombinedChilds($CDFILIAL, $this->combinedConfTela);
        }
        
        $result = [];
        
        foreach ($this->combinedProducts as $combinedProd) {
            // This item serves only as a container. This container holds products that are used to build a combination of products. This is why these products are called combined products.
            $combinedItem = [
                "DSBUTTON"       => $combinedProd['DSBUTTON'], // Button name.
                "PRODUCTS"     => [] // This array will hold the actual combined products, populated below.
            ];

            foreach ($this->combinedChilds as $combinedChild) {
                if (intval($combinedChild['IDTIPOPROD']) == intval($combinedProd['NRBUTTON'])) {
                    $combinedChild['CDFILIAL'] = $CDFILIAL;
                    $combinedChild['CDLOJA']   = $CDLOJA;

                    $combinedItem['PRODUCTS'][$combinedChild['CDARVPROD']] = $combinedChild;
                }
            }
            
            $result[$combinedProd['NRBUTTON']] = $combinedItem;
        }

        return $result;
    }
    
    public function clearCombinedProducts() {
        $this->combinedConfTela = [];
        $this->combinedProducts = [];
        $this->combinedChilds   = [];
    }
    
    public function checkForCombinedProducts($products){
        foreach($products as $product){
            if(isset($product["externalCode"]) && $product["unitPrice"] == 0 && $this->isCombinedProduct($product["options"])){
                return true;
            }
        }
        
        return false;
    }
    
    public function processProduct ($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ) {
        $observations = [];

        // Combos
        if(isset($product["options"]) && !empty($product["options"])) { 
            //Produto combinado
            //verificar se produto combinado ainda vem no mesmo formato
            if(isset($product["externalCode"]) && $product["unitPrice"] == 0 && $this->isCombinedProduct($product["options"])) {
                $this->setHasCombinedProduct(true);
                $processedProduct = $this->processProductAsCombined($generalParams, $product, $order, $ITCOMANDAVEN_SEQ, $this->getCombinedProducts($generalParams['CDFILIAL'], $generalParams['CDCAIXA'], $generalParams['CDLOJA']));

            } else if(isset($product["externalCode"]) || $product["price"] != 0) { //Combo com código externo
                $processedProduct = $this->processProductAsComboWithExtCode($generalParams, $product, $order, $observations, $ITCOMANDAVEN_SEQ);    

            } else { //Combo sem código externo
                $processedProduct = $this->processProductAsComboWithoutExtCode($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);    
            }
        } else {
            $processedProduct = $this->processProductAsSimple($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);
        }

        return $processedProduct;
    }
    
    private function processProductAsCombined ($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ, $combinedProducts) {
        $combinedProductsArr = [];
        $cloneProduct = $product;
        
        for ($i = 0 ; $i < $product["quantity"] ; $i++) {
            
            $massaName    = '';
            $massaPrice   = 0;
            $pizzasPrice  = 0;
            $pizzaflavors = 0;
            
            $tmpSubItens  = [];

            foreach ($product['options'] as $tmpItem) {
                for ($j = 0 ; $j < $tmpItem["quantity"]; $j++) {
                    $tmpItem2 = $tmpItem;
                    $tmpItem2["price"] = ($tmpItem["price"] / $tmpItem["quantity"]);
                    $tmpItem2["quantity"] = 1;
                    array_push($tmpSubItens, $tmpItem2);
                }
            }
            $product['options'] = $tmpSubItens;
            
            foreach ($product['options'] as &$item) {
                $itemDetails = [];
                
                if(strtoupper(substr($item['name'], 0, 5)) === 'MASSA') {
                    //Encontrar elemento que é a massa. Armazenar seu valor.
                    $massaPrice = $item['price'];
                    $massaName = $item['name'];
                } else {
                    //Somar totalPrice de todos os elementos que não são massa.
                    $pizzasPrice += $item['price'];
                    //encontrar a quantidade com base no numero de elementos removendo a massa.
                    $pizzaflavors += $item['quantity'];
                    // Remove caracteres indevidos no primeiro nível
                    $product['externalCode'] = preg_replace("/[^0-9A-Z]/", "", $product['externalCode']);
                    //encontrar cdproduto pelo externalcode do pai (NRBUTTON) e do filho (CDARVPROD)
                    if(isset($combinedProducts[$product['externalCode']])) {
                        if(isset($combinedProducts[$product['externalCode']]['PRODUCTS'])) {
                            // Remove caracteres indevidos no segundo nível
                            $item['externalCode'] = preg_replace("/[^0-9A-Z]/", "", $item['externalCode']);
                            if(isset($combinedProducts[$product['externalCode']]['PRODUCTS'][$item['externalCode']])) {
                                $itemDetails = $combinedProducts[$product['externalCode']]['PRODUCTS'][$item['externalCode']];
                                $item = array_merge($itemDetails, $item);
                            }
                        }
                    }
                    
                    if (empty($itemDetails)) {
                        $this->logCombinedError($generalParams['CDFILIAL'], $generalParams['CDCAIXA'], $generalParams['CDLOJA'], $order);

                        throw new Exception\MenuException("
                            Não existe um produto atrelado ao código externo informado.<br>
                            Produto: {$item['name']}.<br>
                            Código externo: {" . $item['externalCode'] . "}.<br>
                            Verifique no Odhen Retail o código externo deste produto e faça a correção no portal iFood.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
                    }
                }
            }
            
            if(isset($item))
                unset($item);
            
            $pizzasPrice += $massaPrice;
            $fakeQuantity = 0;
            
            foreach ($product['options'] as &$item) {
                
                $item['pizzaOddFlavor'] = false;
                
                if(strtoupper(substr($item['name'], 0, 5)) !== 'MASSA') {
                    //gravar nome do elemento massa em todos os sabores como uma observação digitada.
                    $item['observations'] = ' ' . $massaName. '; ';
                    $item['price'] = $pizzasPrice;
                    $item['quantity'] = round(($item['quantity']/$pizzaflavors), 2);
                    $fakeQuantity += $item['quantity'];
                }
            }
            
            if(isset($item))
                unset($item);
            
            //no caso de 3 sabores um dos itens deve ser 0.34.
            if($fakeQuantity < 1){
                $product['options'][1]['quantity'] += 0.01;
                $product['options'][1]['pizzaOddFlavor'] = true;
            }
            
            foreach ($product['options'] as $item) {
                if(strtoupper(substr($item['name'], 0, 5)) !== 'MASSA') {
                    $item["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                    $ITCOMANDAVEN_SEQ++;
            
                    if(!empty($product['observations'])) {
                        $item["observations"] .= $product['observations'] . '; ';
                    }
                    
                    $productReturn = array(
                        "NMPRODUTOIFOOD" => $item["name"],
                        "CDFILIAL"       => $generalParams["CDFILIAL"],
                        "NRVENDAREST"    => $order["NRVENDAREST"],
                        "NRCOMANDA"      => $order["NRCOMANDA"],
                        "NRPRODCOMVEN"   => $item["NRPRODCOMVEN"],
                        "CDPRODUTO"      => $item["CDPRODUTO"],
                        "QTPRODCOMVEN"   => $item["quantity"],
                        "VRPRECCOMVEN"   => $item["price"], 
                        "TXPRODCOMVEN"   => $item["observations"],
                        "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                        "VRDESCCOMVEN"   => 0,
                        "NRLUGARMESA"    => "01",
                        "IDPRODIMPFIS"   => "N",
                        "CDLOJA"         => $generalParams["CDLOJA"],
                        "VRACRCOMVEN"    => 0,
                        "NRSEQPRODCOM"   => null,
                        "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                        "VRPRECCLCOMVEN" => 0,
                        "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                        "CDPRODPROMOCAO" => null,
                        "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                        "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                        "DSOBSPEDDIGCMD" => null,
                        "CDSENHAPED"     => $order["NRCOMANDAEXT"],
                        "PIZZAODDFLAVOR" => $item['pizzaOddFlavor']
                    );
                    
                    array_push($combinedProductsArr, $productReturn);
                }
            }
            
            $product = $cloneProduct;
        }
        
        return $combinedProductsArr;
    }
    
    private function processProductAsSimple ($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ) {
        $simpleItens = [];
     
        if (!isset($product["externalCode"]) || empty($product["externalCode"]))
            throw new Exception\MenuException("
                O produto {$product['name']} não possui código externo e ele é obrigatório.<br>
                Verifique no Odhen Retail o código externo deste produto e faça a correção no portal iFood.<br>
                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");

        $productDetails = $this->factory->getProduct($product["externalCode"]);
        
        if (empty($productDetails[0]))
            throw new Exception\MenuException("
                Não existe um produto atrelado ao código externo informado.<br>
                Produto: {$product['name']}.<br>
                Código externo: {$product['externalCode']}.<br>
                Verifique no Odhen Retail o código externo deste produto e faça a correção no portal iFood.<br>
                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
            
        $productDetails = array_merge($productDetails[0], $product);

        $productDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
        $ITCOMANDAVEN_SEQ++;

        $userTextObservation = (isset($productDetails["observations"]) && !empty($productDetails["observations"])) ? (strtoupper($this->util->removeAcentos($productDetails['observations']))) : "";
        $productDetails["observations"] = (!empty($userTextObservation)) ? (" $userTextObservation; ") : "";
        $productDetails["DSOBSPEDDIGCMD"] = (!empty($userTextObservation)) ? $userTextObservation : "";

        $productReturn = array(
            "NMPRODUTOIFOOD" => $productDetails["name"],
            "CDFILIAL"       => $generalParams["CDFILIAL"],
            "NRVENDAREST"    => $order["NRVENDAREST"],
            "NRCOMANDA"      => $order["NRCOMANDA"],
            "NRPRODCOMVEN"   => $productDetails["NRPRODCOMVEN"],
            "CDPRODUTO"      => $productDetails["CDPRODUTO"],
            "QTPRODCOMVEN"   => (int)$product["quantity"],
            "VRPRECCOMVEN"   => $productDetails["unitPrice"], 
            "TXPRODCOMVEN"   => $productDetails["observations"],
            "DSOBSPEDDIGCMD" => $productDetails["DSOBSPEDDIGCMD"],
            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
            "VRDESCCOMVEN"   => 0,
            "NRLUGARMESA"    => "01",
            "IDPRODIMPFIS"   => "N",
            "CDLOJA"         => $generalParams["CDLOJA"],
            "VRACRCOMVEN"    => 0,
            "NRSEQPRODCOM"   => null,
            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
            "VRPRECCLCOMVEN" => 0,
            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
            "CDPRODPROMOCAO" => null,
            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
            "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
            "CDSENHAPED"     => $order["NRCOMANDAEXT"],
            "INDEX"          => $product["index"]
        );
        
        array_push($simpleItens, $productReturn);

        return $simpleItens;
    }
    
    private function processProductAsComboWithExtCode($generalParams, $combo, $order, &$observations, &$ITCOMANDAVEN_SEQ) {
        $comboItens = array();

        for ($i = 0 ; $i < $combo["quantity"] ; $i++) {
            
            if(!isset($combo["externalCode"]) || empty($combo["externalCode"]))
                throw new Exception\MenuException("<p align='justify'>
                    O produto {$combo['name']} foi cadastrado com preço e sem código externo. A obrigatoriedade do preço e do código externo dependem das regras abaixo:<br>
                    Caso o produto tenha preço, será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste produto, no formato 'xx.xx.xx.xx.xx'.<br>
                    Caso o produto seja um agrupador, ele não deve ter preço, nem código externo.<br>
                    O preço e código externo (caso tenham) devem ser iguais ao cadastrado previamente no Odhen Retail.<br>
                    Para mais informações, entre em contato com a equipe de suporte da Teknisa.</p>");

            $comboDetails = $this->factory->getProduct($combo["externalCode"]);
            
            if (empty($comboDetails[0]))
                throw new Exception\MenuException("
                    Não existe um produto atrelado ao código externo informado.<br>
                    Produto: {$combo['name']}.<br>
                    Código externo: {$combo['externalCode']}.<br>
                    Verifique no Odhen Retail o código externo deste produto e faça a correção no portal iFood.");

            $comboDetails = array_merge($comboDetails[0], $combo);
            
            $comboDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $productNrSeqProdCom = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 3, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;

            $userTextObservation = (isset($comboDetails["observations"]) && !empty($comboDetails["observations"])) ? (strtoupper($this->util->removeAcentos($comboDetails['observations']))) : "";
            $comboDetails["TXPRODCOMVEN"] = (!empty($userTextObservation)) ? (" $userTextObservation; ") : "";
            $comboDetails["DSOBSPEDDIGCMD"] = (!empty($userTextObservation)) ? $userTextObservation : "";

            $CDPRODPROMO = $comboDetails["CDPRODUTO"];
            
            $itemReturn = array(
                "NMPRODUTOIFOOD"    => $comboDetails["name"],
                "CDFILIAL"          => $generalParams["CDFILIAL"],
                "NRVENDAREST"       => $order["NRVENDAREST"],
                "NRCOMANDA"         => $order["NRCOMANDA"],
                "NRPRODCOMVEN"      => $comboDetails["NRPRODCOMVEN"],
                "CDPRODUTO"         => $comboDetails["CDPRODUTO"],
                "QTPRODCOMVEN"      => 1,
                "VRPRECCOMVEN"      => $comboDetails["unitPrice"],  
                "TXPRODCOMVEN"      => $comboDetails["TXPRODCOMVEN"],
                "DSOBSPEDDIGCMD"    => $comboDetails["DSOBSPEDDIGCMD"],
                "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                "VRDESCCOMVEN"      => 0,
                "NRLUGARMESA"       => "01",
                "IDPRODIMPFIS"      => "N",
                "CDLOJA"            => $generalParams["CDLOJA"],
                "VRACRCOMVEN"       => 0,
                "NRSEQPRODCOM"      => (!empty($comboDetails['optionsPrice']) && $comboDetails['optionsPrice'] >= 0.01) ? $productNrSeqProdCom : null,
                "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"    => 0,
                "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"    => (!empty($comboDetails['optionsPrice']) && $comboDetails['optionsPrice'] >= 0.01) ? $CDPRODPROMO : null,
                "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"       => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                "CDSENHAPED"        => $order["NRCOMANDAEXT"],
                "ITEM_OBSERVATION"  => [],
                "INDEX"             => $combo["index"]
            );

            array_push($comboItens, $itemReturn);

            foreach ($combo["options"] as $item) {
                //========================== Observações Sem Preço ==========================
                if(isset($item['price']) && !$item['price'] && !empty($item['name'])) {
                    // Observações tipo O e R
                    
                    if(!isset($item['externalCode']) || empty($item['externalCode']))
                        throw new Exception\MenuException("
                            O subitem {$item['name']} foi cadastrado sem preço e sem código externo. Apesar do preço não ser uma exigência (ver abaixo), o código externo é obrigatório para todos os subitems.<br>
                            Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                            Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'xx.xx.xx.xx.xx'.<br>
                            O código externo deve ser igual ao cadastrado previamente no Retail.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do iFood.");

                    $observationCode = $item['externalCode'];

                    if(!strpos($observationCode, "|"))
                        throw new Exception\MenuException("
                            O subitem {$item['name']} foi cadastrado sem preço e não existe um subitem cadastrado no Retail com o código externo informado: {$observationCode}.<br>
                            Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código externo deste subitem. O formato correto é Código Grupo de Observação|Código de Observação (xx|yy).<br>
                            Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                            O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do iFood.");

                    $validateObs = $this->factory->getObservation($observationCode);

                    if(!count($validateObs))
                        throw new Exception\MenuException("
                            Não existe uma observação atrelada ao código externo informado.<br>
                            Item do pedido: {$item['name']}.<br>
                            Código externo: {$item['externalCode']}.<br>
                            Verifique no Odhen Retail o código externo desta observação e faça a correção no portal iFood.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");

                    array_push($comboItens[0]['ITEM_OBSERVATION'], [
                        'CDFILIAL'        => $generalParams["CDFILIAL"],  
                        'NRVENDAREST'     => $order["NRVENDAREST"], 
                        'NRCOMANDA'       => $order["NRCOMANDA"], 
                        'NRPRODCOMVEN'    => $comboDetails["NRPRODCOMVEN"],  
                        'CDGRPOCOR'       => $validateObs['CDGRPOCOR'],   
                        'CDOCORR'         => $validateObs['CDOCORR'],  
                        'NRPRODCOMVENOBS' => null,  
                        'QTPRODCOMVENOBS' => $item['quantity'],
                        'INDEX'           => $item['index']
                    ]);

                    // Faz referencia na TXPRODCOMVEN do item Pai
                    $this->processObservationWithoutPrice($comboDetails["NRPRODCOMVEN"], $validateObs['DSOCORR'], $item['quantity'], $comboItens);
                    continue;                    
                }
                //======================= Fim de Observações Sem Preço ======================
                
                $itemDetails = $this->factory->getProduct($item["externalCode"]);
                
                if (empty($itemDetails[0])){
                    if(!isset($item['externalCode']) || empty($item['externalCode'])) {
                        throw new Exception\MenuException("
                            O subitem {$item['name']} foi cadastrado sem código externo. O código externo é obrigatório para todos os subitems.<br>
                            Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                            Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'xx.xx.xx.xx.xx'.<br>
                            O código externo deve ser igual ao cadastrado previamente no Retail.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do iFood.");   
                    }
                            
                    else if(strpos($item['externalCode'], "|")){
                        $itemDetails = $this->factory->getProductByObsCode($item["externalCode"]);
                        
                        $validateObs = $this->factory->getObservation($item["externalCode"]);

                        if(!count($validateObs))
                            throw new Exception\MenuException("
                                Não existe uma observação atrelada ao código externo informado.<br>
                                Item do pedido: {$item['name']}.<br>
                                Código externo: {$item['externalCode']}.<br>
                                Verifique no Odhen Retail o código externo desta observação e faça a correção no portal iFood.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
    
                        array_push($comboItens[0]['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],  
                            'NRVENDAREST'     => $order["NRVENDAREST"], 
                            'NRCOMANDA'       => $order["NRCOMANDA"], 
                            'NRPRODCOMVEN'    => $comboDetails["NRPRODCOMVEN"],  
                            'CDGRPOCOR'       => $validateObs['CDGRPOCOR'],   
                            'CDOCORR'         => $validateObs['CDOCORR'],  
                            'NRPRODCOMVENOBS' => null,  
                            'QTPRODCOMVENOBS' => $item['quantity'],
                            'INDEX'           => $item['index']
                        ]);

                        // Faz referencia na TXPRODCOMVEN do item Pai
                        // $this->processObservationWithoutPrice($comboDetails["NRPRODCOMVEN"], $validateObs['DSOCORR'], $item['quantity'], $comboItens);
                        
                        /*throw new Exception\MenuException("
                            O subitem '{$item['name']}' possui preço ({$item['price']}) e o código externo ({$item['externalCode']}) está no formato de uma observação que não possui preço (XX|YY).<br>
                            Caso o subitem seja uma observação ou remoção (por exemplo, 'Sem salada' ou 'Ao ponto'), você deve remover o preço deste subitem.<br>
                            Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), você deve corrigir o código externo deste subitem, no formato 'xx.xx.xx.xx.xx'.<br>
                            O código externo e o valor do subitem devem ser iguais ao cadastrado previamente no Odhen Retail.<br>
                            Verifique no Retail o código externo e o valor desta observação e faça a correção no portal iFood.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do iFood.");*/
                    }
                    
                    else{
                        throw new Exception\MenuException("
                            Não existe um produto atrelado ao código externo informado.<br>
                            Produto: {$item['name']}.<br>
                            Código externo: {$item['externalCode']}.<br>
                            Verifique no Retail o código externo e o valor desta observação e faça a correção no portal iFood.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do iFood.");   
                    }
                }
                    
                $itemDetails = array_merge($itemDetails[0], $item);
                
                $userTextObservation = (isset($itemDetails["observations"]) && !empty($itemDetails["observations"])) ? (strtoupper($this->util->removeAcentos($itemDetails['observations']))) : "";
                $itemDetails["TXPRODCOMVEN"] = (!empty($userTextObservation)) ? (" $userTextObservation; ") : "";
                $itemDetails["DSOBSPEDDIGCMD"] = (!empty($userTextObservation)) ? $userTextObservation : "";

                for ($y = 0 ; $y < $itemDetails["quantity"] ; $y++){
                    $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                    $ITCOMANDAVEN_SEQ++;
                    
                    $itemReturn = array(
                        "NMPRODUTOIFOOD" => $itemDetails["name"],
                        "CDFILIAL"       => $generalParams["CDFILIAL"],
                        "NRVENDAREST"    => $order["NRVENDAREST"],
                        "NRCOMANDA"      => $order["NRCOMANDA"],
                        "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                        "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                        "QTPRODCOMVEN"   => 1,
                        "VRPRECCOMVEN"   => $itemDetails["unitPrice"],  
                        "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                        "DSOBSPEDDIGCMD" => $itemDetails["DSOBSPEDDIGCMD"],
                        "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                        "VRDESCCOMVEN"   => 0,
                        "NRLUGARMESA"    => "01",
                        "IDPRODIMPFIS"   => "N",
                        "CDLOJA"         => $generalParams["CDLOJA"],
                        "VRACRCOMVEN"    => 0,
                        "NRSEQPRODCOM"   => $productNrSeqProdCom,
                        "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                        "VRPRECCLCOMVEN" => 0,
                        "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                        "CDPRODPROMOCAO" => $CDPRODPROMO,
                        "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                        "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                        "CDSENHAPED"     => $order["NRCOMANDAEXT"],
                        "INDEX"          => $item["index"]
                    );
                    
                    array_push($comboItens, $itemReturn);   
                }
            }
        }
        
        return $comboItens;
    }

    private function processProductAsComboWithoutExtCode($generalParams, $combo, $order, &$ITCOMANDAVEN_SEQ) {
        $comboItens = array();

        for ($i = 0 ; $i < $combo["quantity"]; $i++) {

            foreach ($combo["options"] as $item) {
                $itemDetails = $this->factory->getProduct($item["externalCode"]);
                
                if (empty($itemDetails[0]))
                    throw new Exception\MenuException("
                        Não existe um produto atrelado ao código externo informado.<br>
                        Produto: {$item['name']}.<br>
                        Código externo: {$item['externalCode']}.<br>
                        Verifique no Odhen Retail o código externo deste produto e faça a correção no portal iFood.<br>
                        Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do iFood.");
                    
                $itemDetails = array_merge($itemDetails[0], $item);
                
                $userTextObservation = (isset($itemDetails["observations"]) && !empty($itemDetails["observations"])) ? (strtoupper($this->util->removeAcentos($itemDetails['observations']))) : "";
                $itemDetails["TXPRODCOMVEN"] = (!empty($userTextObservation)) ? (" $userTextObservation; ") : "";
                $itemDetails["DSOBSPEDDIGCMD"] = (!empty($userTextObservation)) ? $userTextObservation : "";

                if(!empty($combo['observations'])) {

                    if(empty($itemDetails['TXPRODCOMVEN'])) {
                        $itemDetails['TXPRODCOMVEN'] = " ";                        
                    }

                    if(empty($itemDetails["DSOBSPEDDIGCMD"])) {
                        $itemDetails["DSOBSPEDDIGCMD"] = "";
                    }

                    $itemDetails['TXPRODCOMVEN']   .= (strtoupper($this->util->removeAcentos($combo['observations']))) . "; ";
                    $itemDetails["DSOBSPEDDIGCMD"] .= (strtoupper($this->util->removeAcentos($combo['observations'])));
                }
                
                for ($y = 0 ; $y < $itemDetails["quantity"] ; $y++){
                    $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                    $ITCOMANDAVEN_SEQ++;
                    
                    $itemReturn = array(
                        "NMPRODUTOIFOOD" => $itemDetails["name"],
                        "CDFILIAL"       => $generalParams["CDFILIAL"],
                        "NRVENDAREST"    => $order["NRVENDAREST"],
                        "NRCOMANDA"      => $order["NRCOMANDA"],
                        "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                        "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                        "QTPRODCOMVEN"   => 1,
                        "VRPRECCOMVEN"   => $itemDetails["unitPrice"],  
                        "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                        "DSOBSPEDDIGCMD" => $itemDetails["DSOBSPEDDIGCMD"],
                        "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                        "VRDESCCOMVEN"   => 0,
                        "NRLUGARMESA"    => "01",
                        "IDPRODIMPFIS"   => "N",
                        "CDLOJA"         => $generalParams["CDLOJA"],
                        "VRACRCOMVEN"    => 0,
                        "NRSEQPRODCOM"   => null,
                        "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                        "VRPRECCLCOMVEN" => 0,
                        "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                        "CDPRODPROMOCAO" => null,
                        "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                        "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                        "CDSENHAPED"     => $order["NRCOMANDAEXT"],
                        "INDEX"          => $item["index"]
                    );
                    array_push($comboItens, $itemReturn);
                }
            }
        }

        return $comboItens;
    }

    private function processObservationWithoutPrice($nrProdComVenKey, $observation, $quantity, &$products) {
        foreach($products as &$product) {
            if($nrProdComVenKey != $product['NRPRODCOMVEN']) {
                continue;
            }

            for($i = 0; $i < $quantity; $i++) {
                if(empty($product["TXPRODCOMVEN"])) {
                    $product["TXPRODCOMVEN"] = " $observation; ";
                } else {
                    $product["TXPRODCOMVEN"] .= "$observation; ";
                }
            }            
        }
        
        if(isset($product)) {
            unset($product);
        }
    }

    public function insertProduct ($product, $consumerName = null) {
        $this->factory->insertProduct($product, $consumerName);
    }
    
    public function insertObservation($observation) {
        $this->factory->insertObservation($observation);
    }
    
    public function logCombinedError($CDFILIAL, $CDCAIXA, $CDLOJA, $order) {
        $combinedLog = [
            "CDFILIAL" => $CDFILIAL,
            "CDCAIXA" => $CDCAIXA,
            "CDLOJA" => $CDLOJA,
            "NRCONFTELA" => $this->combinedConfTela,
            "combinedProducts" => $this->combinedProducts,
            "combinedChilds" => $this->combinedChilds
        ];
        
        $f = $CDFILIAL."-".$order["displayId"]."-".round(microtime(true) * 1000).".txt";
        $this->util->writeLog('combinedProdError', $f, json_encode($combinedLog, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }
}
